@@ -24,6 +24,8 @@
 #ifndef __GNOME_DRUID_PAGE_STANDARD_H__
 #define __GNOME_DRUID_PAGE_STANDARD_H__
 
+#ifndef GNOME_DISABLE_DEPRECATED
+
 #include <gtk/gtk.h>
 #include "gnome-druid-page.h"
 
@@ -41,6 +43,20 @@ typedef struct _GnomeDruidPageStandard        GnomeDruidPageStandard;
 typedef struct _GnomeDruidPageStandardPrivate GnomeDruidPageStandardPrivate;
 typedef struct _GnomeDruidPageStandardClass   GnomeDruidPageStandardClass;
 
+/**
+ * GnomeDruidPageStandard
+ * @vbox: A packing widget that holds the contents of the page.
+ * @title: The title of the displayed page.
+ * @logo: The logo of the displayed page.
+ * @top_watermark: The watermark at the top of the displated page.
+ * @title_foreground: The color of the title text.
+ * @background: The color of the background of the top section and title.
+ * @logo_background: The background color of the logo.
+ * @contents_background: The background color of the contents section.
+ *
+ * A widget representing pages that are not initial or terminal pages of a
+ * druid.
+ */
 struct _GnomeDruidPageStandard
 {
 	GnomeDruidPage parent;
@@ -61,26 +77,22 @@ struct _GnomeDruidPageStandard
 struct _GnomeDruidPageStandardClass
 {
 	GnomeDruidPageClass parent_class;
-};
 
-#ifndef GNOME_DISABLE_DEPRECATED
+	/* Padding for possible expansion */
+	gpointer padding1;
+	gpointer padding2;
+};
 
 #define gnome_druid_page_standard_set_bg_color      gnome_druid_page_standard_set_background
 #define gnome_druid_page_standard_set_logo_bg_color gnome_druid_page_standard_set_logo_background
 #define gnome_druid_page_standard_set_title_color   gnome_druid_page_standard_set_title_foreground
 
-#endif
-
 
 GType      gnome_druid_page_standard_get_type                (void) G_GNUC_CONST;
 GtkWidget *gnome_druid_page_standard_new                     (void);
 GtkWidget *gnome_druid_page_standard_new_with_vals           (const gchar            *title,
 							      GdkPixbuf              *logo,
 							      GdkPixbuf              *top_watermark);
-void       gnome_druid_page_standard_construct               (GnomeDruidPageStandard *druid_page_standard,
-							      const gchar	    *title,
-							      GdkPixbuf		    *logo,
-							      GdkPixbuf              *top_watermark);
 
 void       gnome_druid_page_standard_set_title               (GnomeDruidPageStandard *druid_page_standard,
 							      const gchar            *title);
@@ -99,11 +111,13 @@ void       gnome_druid_page_standard_set_contents_background (GnomeDruidPageStan
 
 /* Convenience Function */
 void       gnome_druid_page_standard_append_item             (GnomeDruidPageStandard *druid_page_standard,
-							      const gchar            *question_mnemonic,
+							      const gchar            *question,
 							      GtkWidget              *item,
-							      const gchar            *additional_info_markup);
+							      const gchar            *additional_info);
 
 G_END_DECLS
 
+#endif /* GNOME_DISABLE_DEPRECATED */
+
 #endif /* __GNOME_DRUID_PAGE_STANDARD_H__ */
 
