@@ -36,6 +36,8 @@
 #include <glib.h>
 #include <stdio.h>
 
+#include <glib/gchecksum.h>
+
 #define GDK_PIXBUF_ENABLE_BACKEND
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
@@ -51,14 +53,6 @@
 
 #define SECONDS_BETWEEN_STATS 10
 
-struct ThumbMD5Context {
-	guint32 buf[4];
-	guint32 bits[2];
-	unsigned char in[64];
-};
-
-static void thumb_md5 (const char *string, unsigned char digest[16]);
-
 struct _GnomeThumbnailFactoryPrivate {
   char *application;
   GnomeThumbnailSize size;
@@ -90,63 +84,6 @@ GNOME_CLASS_BOILERPLATE (GnomeThumbnailFactory,
 static void gnome_thumbnail_factory_instance_init (GnomeThumbnailFactory      *factory);
 static void gnome_thumbnail_factory_class_init    (GnomeThumbnailFactoryClass *class);
 
-#if !GLIB_CHECK_VERSION (2,7,0)
-/* g_chmod lifted from GLib */
-#ifndef G_OS_WIN32
-#define g_chmod(filename, mode) chmod (filename, mode)
-#else
-static int
-g_chmod (const gchar *filename,
-	 int          mode)
-{
-#ifdef G_OS_WIN32
-  if (G_WIN32_HAVE_WIDECHAR_API ())
-    {
-      wchar_t *wfilename = g_utf8_to_utf16 (filename, -1, NULL, NULL, NULL);
-      int retval;
-      int save_errno;
-      
-      if (wfilename == NULL)
-	{
-	  errno = EINVAL;
-	  return -1;
-	}
-
-      retval = _wchmod (wfilename, mode);
-      save_errno = errno;
-
-      g_free (wfilename);
-
-      errno = save_errno;
-      return retval;
-    }
-  else
-    {    
-      gchar *cp_filename = g_locale_from_utf8 (filename, -1, NULL, NULL, NULL);
-      int retval;
-      int save_errno;
-
-      if (cp_filename == NULL)
-	{
-	  errno = EINVAL;
-	  return -1;
-	}
-
-      retval = chmod (cp_filename, mode);
-      save_errno = errno;
-
-      g_free (cp_filename);
-
-      errno = save_errno;
-      return retval;
-    }
-#else
-  return chmod (filename, mode);
-#endif
-}
-#endif	/* Win32 */
-#endif	/* GLib < 2.7.0 */
-
 static void
 thumbnail_info_free (gpointer data)
 {
@@ -444,25 +381,6 @@ gnome_thumbnail_factory_new (GnomeThumbnailSize size)
   return factory;
 }
 
-static char *
-thumb_digest_to_ascii (unsigned char digest[16])
-{
-  static const char hex_digits[] = "0123456789abcdef";
-  char *res;
-  int i;
-  
-  res = g_malloc (33);
-  
-  for (i = 0; i < 16; i++) {
-    res[2*i] = hex_digits[digest[i] >> 4];
-    res[2*i+1] = hex_digits[digest[i] & 0xf];
-  }
-  
-  res[32] = 0;
-  
-  return res;
-}
-
 static void
 thumb_digest_from_ascii (unsigned char *ascii, unsigned char digest[16])
 {
@@ -649,23 +567,27 @@ gnome_thumbnail_factory_lookup (GnomeThumbnailFactory *factory,
 				time_t                 mtime)
 {
   GnomeThumbnailFactoryPrivate *priv = factory->priv;
-  unsigned char digest[16];
-  char *path, *md5, *file;
+  char *path, *file;
   gpointer value;
   struct ThumbnailInfo *info;
+  GChecksum *checksum;
+  guint8 digest[16];
+  gsize digest_len = sizeof (digest);
+
+  checksum = g_checksum_new (G_CHECKSUM_MD5);
+  g_checksum_update (checksum, (const guchar *) uri, strlen (uri));
+
+  g_checksum_get_digest (checksum, digest, &digest_len);
+  g_assert (digest_len == 16);
 
   g_mutex_lock (priv->lock);
 
   gnome_thumbnail_factory_ensure_uptodate (factory);
 
-  thumb_md5 (uri, digest);
-
   if (g_hash_table_lookup_extended (priv->existing_thumbs,
 				    digest, NULL, &value))
     {
-      md5 = thumb_digest_to_ascii (digest);
-      file = g_strconcat (md5, ".png", NULL);
-      g_free (md5);
+      file = g_strconcat (g_checksum_get_string (checksum), ".png", NULL);
   
       path = g_build_filename (g_get_home_dir (),
 			       ".thumbnails",
@@ -679,10 +601,9 @@ gnome_thumbnail_factory_lookup (GnomeThumbnailFactory *factory,
 	  info = load_thumbnail_info (path);
 	  if (info)
 	    {
-	      unsigned char *key;
-	      key = g_malloc (16);
-	      memcpy (key, digest, 16);
-	      g_hash_table_insert (priv->existing_thumbs, key, info);
+	      g_hash_table_insert (priv->existing_thumbs,
+                                   g_memdup (digest, sizeof (digest)),
+                                   info);
 	    }
 	}
       else
@@ -693,6 +614,7 @@ gnome_thumbnail_factory_lookup (GnomeThumbnailFactory *factory,
 	  strcmp (info->uri, uri) == 0)
 	{
 	  g_mutex_unlock (priv->lock);
+          g_checksum_free (checksum);
 	  return path;
 	}
       
@@ -701,6 +623,8 @@ gnome_thumbnail_factory_lookup (GnomeThumbnailFactory *factory,
 
   g_mutex_unlock (priv->lock);
   
+  g_checksum_free (checksum);
+
   return NULL;
 }
 
@@ -726,10 +650,18 @@ gnome_thumbnail_factory_has_valid_failed_thumbnail (GnomeThumbnailFactory *facto
 						    time_t                 mtime)
 {
   GnomeThumbnailFactoryPrivate *priv = factory->priv;
-  unsigned char digest[16];
-  char *path, *file, *md5;
+  char *path, *file;
   GdkPixbuf *pixbuf;
   gboolean res;
+  GChecksum *checksum;
+  guint8 digest[16];
+  gsize digest_len = sizeof (digest);
+
+  checksum = g_checksum_new (G_CHECKSUM_MD5);
+  g_checksum_update (checksum, (const guchar *) uri, strlen (uri));
+
+  g_checksum_get_digest (checksum, digest, &digest_len);
+  g_assert (digest_len == 16);
 
   res = FALSE;
 
@@ -737,14 +669,10 @@ gnome_thumbnail_factory_has_valid_failed_thumbnail (GnomeThumbnailFactory *facto
 
   gnome_thumbnail_factory_ensure_failed_uptodate (factory);
   
-  thumb_md5 (uri, digest);
-  
   if (g_hash_table_lookup_extended (factory->priv->failed_thumbs,
 				    digest, NULL, NULL))
     {
-        md5 = thumb_digest_to_ascii (digest);
-	file = g_strconcat (md5, ".png", NULL);
-	g_free (md5);
+	file = g_strconcat (g_checksum_get_string (checksum), ".png", NULL);
 
 	path = g_build_filename (g_get_home_dir (),
 				 ".thumbnails/fail",
@@ -764,7 +692,9 @@ gnome_thumbnail_factory_has_valid_failed_thumbnail (GnomeThumbnailFactory *facto
     }
 
   g_mutex_unlock (priv->lock);
-  
+
+  g_checksum_free (checksum);
+
   return res;
 }
 
@@ -1148,8 +1078,7 @@ gnome_thumbnail_factory_save_thumbnail (GnomeThumbnailFactory *factory,
 					time_t                 original_mtime)
 {
   GnomeThumbnailFactoryPrivate *priv = factory->priv;
-  unsigned char *digest;
-  char *path, *md5, *file, *dir;
+  char *path, *file, *dir;
   char *tmp_path;
   const char *width, *height;
   int tmp_fd;
@@ -1157,19 +1086,23 @@ gnome_thumbnail_factory_save_thumbnail (GnomeThumbnailFactory *factory,
   gboolean saved_ok;
   struct stat statbuf;
   struct ThumbnailInfo *info;
-  
+  GChecksum *checksum;
+  guint8 digest[16];
+  gsize digest_len = sizeof (digest);
+
+  checksum = g_checksum_new (G_CHECKSUM_MD5);
+  g_checksum_update (checksum, (const guchar *) uri, strlen (uri));
+
+  g_checksum_get_digest (checksum, digest, &digest_len);
+  g_assert (digest_len == 16);
+
   g_mutex_lock (priv->lock);
   
   gnome_thumbnail_factory_ensure_uptodate (factory);
 
   g_mutex_unlock (priv->lock);
-  
-  digest = g_malloc (16);
-  thumb_md5 (uri, digest);
 
-  md5 = thumb_digest_to_ascii (digest);
-  file = g_strconcat (md5, ".png", NULL);
-  g_free (md5);
+  file = g_strconcat (g_checksum_get_string (checksum), ".png", NULL);
   
   dir = g_build_filename (g_get_home_dir (),
 			  ".thumbnails",
@@ -1198,7 +1131,6 @@ gnome_thumbnail_factory_save_thumbnail (GnomeThumbnailFactory *factory,
       g_free (dir);
       g_free (tmp_path);
       g_free (path);
-      g_free (digest);
       return;
     }
   close (tmp_fd);
@@ -1238,7 +1170,9 @@ gnome_thumbnail_factory_save_thumbnail (GnomeThumbnailFactory *factory,
       
       g_mutex_lock (priv->lock);
       
-      g_hash_table_insert (factory->priv->existing_thumbs, digest, info);
+      g_hash_table_insert (factory->priv->existing_thumbs,
+                           g_memdup (digest, sizeof (digest)),
+                           info);
       /* Make sure we don't re-read the directory. We should be uptodate
        * with all previous changes du to the ensure_uptodate above.
        * There is still a small window here where we might miss exisiting
@@ -1252,7 +1186,6 @@ gnome_thumbnail_factory_save_thumbnail (GnomeThumbnailFactory *factory,
     }
   else
     {
-      g_free (digest);
       gnome_thumbnail_factory_create_failed_thumbnail (factory, uri, original_mtime);
     }
 
@@ -1280,27 +1213,30 @@ gnome_thumbnail_factory_create_failed_thumbnail (GnomeThumbnailFactory *factory,
 						 time_t                 mtime)
 {
   GnomeThumbnailFactoryPrivate *priv = factory->priv;
-  unsigned char *digest;
-  char *path, *md5, *file, *dir;
+  char *path, *file, *dir;
   char *tmp_path;
   int tmp_fd;
   char mtime_str[21];
   gboolean saved_ok;
   struct stat statbuf;
   GdkPixbuf *pixbuf;
-  
+  GChecksum *checksum;
+  guint8 digest[16];
+  gsize digest_len = sizeof (digest);
+
+  checksum = g_checksum_new (G_CHECKSUM_MD5);
+  g_checksum_update (checksum, (const guchar *) uri, strlen (uri));
+
+  g_checksum_get_digest (checksum, digest, &digest_len);
+  g_assert (digest_len == 16);
+
   g_mutex_lock (priv->lock);
   
   gnome_thumbnail_factory_ensure_failed_uptodate (factory);
 
   g_mutex_unlock (priv->lock);
 
-  digest = g_malloc (16);
-  thumb_md5 (uri, digest);
-
-  md5 = thumb_digest_to_ascii (digest);
-  file = g_strconcat (md5, ".png", NULL);
-  g_free (md5);
+  file = g_strconcat (g_checksum_get_string (checksum), ".png", NULL);
   
   dir = g_build_filename (g_get_home_dir (),
 			  ".thumbnails/fail",
@@ -1328,7 +1264,6 @@ gnome_thumbnail_factory_create_failed_thumbnail (GnomeThumbnailFactory *factory,
       g_free (dir);
       g_free (tmp_path);
       g_free (path);
-      g_free (digest);
       return;
     }
   close (tmp_fd);
@@ -1350,7 +1285,9 @@ gnome_thumbnail_factory_create_failed_thumbnail (GnomeThumbnailFactory *factory,
 
       g_mutex_lock (priv->lock);
       
-      g_hash_table_insert (factory->priv->failed_thumbs, digest, NULL);
+      g_hash_table_insert (factory->priv->failed_thumbs,
+                           g_memdup (digest, sizeof (digest)),
+                           NULL);
       /* Make sure we don't re-read the directory. We should be uptodate
        * with all previous changes du to the ensure_uptodate above.
        * There is still a small window here where we might miss exisiting
@@ -1362,8 +1299,6 @@ gnome_thumbnail_factory_create_failed_thumbnail (GnomeThumbnailFactory *factory,
       
       g_mutex_unlock (priv->lock);
     }
-  else
-    g_free (digest);
 
   g_free (dir);
   g_free (path);
@@ -1380,14 +1315,15 @@ gnome_thumbnail_factory_create_failed_thumbnail (GnomeThumbnailFactory *factory,
  * Return value: A string with the MD5 digest of the uri string.
  *
  * Since: 2.2
+ *
+ * @Deprecated: 2.22: Use #GChecksum instead
  **/
 char *
 gnome_thumbnail_md5 (const char *uri)
 {
-  unsigned char digest[16];
-
-  thumb_md5 (uri, digest);
-  return thumb_digest_to_ascii (digest);
+  return g_compute_checksum_for_data (G_CHECKSUM_MD5,
+                                      (const guchar *) uri,
+                                      strlen (uri));
 }
 
 /**
@@ -1485,279 +1421,3 @@ gnome_thumbnail_is_valid (GdkPixbuf          *pixbuf,
   
   return TRUE;
 }
-
-
-/*
- * This code implements the MD5 message-digest algorithm.
- * The algorithm is due to Ron Rivest.  This code was
- * written by Colin Plumb in 1993, no copyright is claimed.
- * This code is in the public domain; do with it what you wish.
- *
- * Equivalent code is available from RSA Data Security, Inc.
- * This code has been tested against that, and is equivalent,
- * except that you don't need to include two pages of legalese
- * with every copy.
- *
- * To compute the message digest of a chunk of bytes, declare an
- * ThumbMD5Context structure, pass it to thumb_md5_init, call
- * thumb_md5_update as needed on buffers full of bytes, and then call
- * thumb_md5_final, which will fill a supplied 32-byte array with the
- * digest in ascii form. 
- *
- */
-
-static void thumb_md5_init      (struct ThumbMD5Context *context);
-static void thumb_md5_update    (struct ThumbMD5Context *context,
-				 unsigned char const    *buf,
-				 unsigned                len);
-static void thumb_md5_final     (unsigned char           digest[16],
-				 struct ThumbMD5Context *context);
-static void thumb_md5_transform (guint32                 buf[4],
-				 guint32 const           in[16]);
-
-
-static void
-thumb_md5 (const char *string, unsigned char digest[16])
-{
-  struct ThumbMD5Context md5_context;
-  
-  thumb_md5_init (&md5_context);
-  thumb_md5_update (&md5_context, (unsigned char *)string, strlen (string));
-  thumb_md5_final (digest, &md5_context);
-}
-
-#if G_BYTE_ORDER == G_LITTLE_ENDIAN
-#define byteReverse(buf, len)	/* Nothing */
-#else
-
-/*
- * Note: this code is harmless on little-endian machines.
- */
-static void
-byteReverse(unsigned char *buf, unsigned longs)
-{
-    guint32 t;
-    do {
-	t = (guint32) ((unsigned) buf[3] << 8 | buf[2]) << 16 |
-	    ((unsigned) buf[1] << 8 | buf[0]);
-	*(guint32 *) buf = t;
-	buf += 4;
-    } while (--longs);
-}
-
-#endif
-
-/*
- * Start MD5 accumulation.  Set bit count to 0 and buffer to mysterious
- * initialization constants.
- */
-static void 
-thumb_md5_init (struct ThumbMD5Context *ctx)
-{
-    ctx->buf[0] = 0x67452301;
-    ctx->buf[1] = 0xefcdab89;
-    ctx->buf[2] = 0x98badcfe;
-    ctx->buf[3] = 0x10325476;
-
-    ctx->bits[0] = 0;
-    ctx->bits[1] = 0;
-}
-
-/*
- * Update context to reflect the concatenation of another buffer full
- * of bytes.
- */
-static void 
-thumb_md5_update (struct ThumbMD5Context *ctx,
-		  unsigned char const *buf,
-		  unsigned len)
-{
-    guint32 t;
-
-    /* Update bitcount */
-
-    t = ctx->bits[0];
-    if ((ctx->bits[0] = t + ((guint32) len << 3)) < t)
-	ctx->bits[1]++;		/* Carry from low to high */
-    ctx->bits[1] += len >> 29;
-
-    t = (t >> 3) & 0x3f;	/* Bytes already in shsInfo->data */
-
-    /* Handle any leading odd-sized chunks */
-
-    if (t) {
-	unsigned char *p = (unsigned char *) ctx->in + t;
-
-	t = 64 - t;
-	if (len < t) {
-	    memcpy (p, buf, len);
-	    return;
-	}
-	memcpy (p, buf, t);
-	byteReverse (ctx->in, 16);
-	thumb_md5_transform (ctx->buf, (guint32 *) ctx->in);
-	buf += t;
-	len -= t;
-    }
-
-    /* Process data in 64-byte chunks */
-
-    while (len >= 64) {
-	memcpy (ctx->in, buf, 64);
-	byteReverse (ctx->in, 16);
-	thumb_md5_transform (ctx->buf, (guint32 *) ctx->in);
-	buf += 64;
-	len -= 64;
-    }
-
-    /* Handle any remaining bytes of data. */
-
-    memcpy(ctx->in, buf, len);
-}
-
-/*
- * Final wrapup - pad to 64-byte boundary with the bit pattern 
- * 1 0* (64-bit count of bits processed, MSB-first)
- */
-static void 
-thumb_md5_final (unsigned char digest[16], struct ThumbMD5Context *ctx)
-{
-    unsigned count;
-    unsigned char *p;
-
-    /* Compute number of bytes mod 64 */
-    count = (ctx->bits[0] >> 3) & 0x3F;
-
-    /* Set the first char of padding to 0x80.  This is safe since there is
-       always at least one byte free */
-    p = ctx->in + count;
-    *p++ = 0x80;
-
-    /* Bytes of padding needed to make 64 bytes */
-    count = 64 - 1 - count;
-
-    /* Pad out to 56 mod 64 */
-    if (count < 8) {
-	/* Two lots of padding:  Pad the first block to 64 bytes */
-	memset (p, 0, count);
-	byteReverse (ctx->in, 16);
-	thumb_md5_transform (ctx->buf, (guint32 *) ctx->in);
-
-	/* Now fill the next block with 56 bytes */
-	memset(ctx->in, 0, 56);
-    } else {
-	/* Pad block to 56 bytes */
-	memset(p, 0, count - 8);
-    }
-    byteReverse(ctx->in, 14);
-
-    /* Append length in bits and transform */
-    ((guint32 *) ctx->in)[14] = ctx->bits[0];
-    ((guint32 *) ctx->in)[15] = ctx->bits[1];
-
-    thumb_md5_transform (ctx->buf, (guint32 *) ctx->in);
-    byteReverse ((unsigned char *) ctx->buf, 4);
-    memcpy (digest, ctx->buf, 16);
-    memset (ctx, 0, sizeof(ctx));	/* In case it's sensitive */
-}
-
-
-/* The four core functions - F1 is optimized somewhat */
-
-#define F1(x, y, z) (z ^ (x & (y ^ z)))
-#define F2(x, y, z) F1 (z, x, y)
-#define F3(x, y, z) (x ^ y ^ z)
-#define F4(x, y, z) (y ^ (x | ~z))
-
-/* This is the central step in the MD5 algorithm. */
-#define thumb_md5_step(f, w, x, y, z, data, s) \
-	( w += f(x, y, z) + data,  w = w<<s | w>>(32-s),  w += x )
-
-/*
- * The core of the MD5 algorithm, this alters an existing MD5 hash to
- * reflect the addition of 16 longwords of new data.  ThumbMD5Update blocks
- * the data and converts bytes into longwords for this routine.
- */
-static void 
-thumb_md5_transform (guint32 buf[4], guint32 const in[16])
-{
-    register guint32 a, b, c, d;
-
-    a = buf[0];
-    b = buf[1];
-    c = buf[2];
-    d = buf[3];
-
-    thumb_md5_step(F1, a, b, c, d, in[0] + 0xd76aa478, 7);
-    thumb_md5_step(F1, d, a, b, c, in[1] + 0xe8c7b756, 12);
-    thumb_md5_step(F1, c, d, a, b, in[2] + 0x242070db, 17);
-    thumb_md5_step(F1, b, c, d, a, in[3] + 0xc1bdceee, 22);
-    thumb_md5_step(F1, a, b, c, d, in[4] + 0xf57c0faf, 7);
-    thumb_md5_step(F1, d, a, b, c, in[5] + 0x4787c62a, 12);
-    thumb_md5_step(F1, c, d, a, b, in[6] + 0xa8304613, 17);
-    thumb_md5_step(F1, b, c, d, a, in[7] + 0xfd469501, 22);
-    thumb_md5_step(F1, a, b, c, d, in[8] + 0x698098d8, 7);
-    thumb_md5_step(F1, d, a, b, c, in[9] + 0x8b44f7af, 12);
-    thumb_md5_step(F1, c, d, a, b, in[10] + 0xffff5bb1, 17);
-    thumb_md5_step(F1, b, c, d, a, in[11] + 0x895cd7be, 22);
-    thumb_md5_step(F1, a, b, c, d, in[12] + 0x6b901122, 7);
-    thumb_md5_step(F1, d, a, b, c, in[13] + 0xfd987193, 12);
-    thumb_md5_step(F1, c, d, a, b, in[14] + 0xa679438e, 17);
-    thumb_md5_step(F1, b, c, d, a, in[15] + 0x49b40821, 22);
-		
-    thumb_md5_step(F2, a, b, c, d, in[1] + 0xf61e2562, 5);
-    thumb_md5_step(F2, d, a, b, c, in[6] + 0xc040b340, 9);
-    thumb_md5_step(F2, c, d, a, b, in[11] + 0x265e5a51, 14);
-    thumb_md5_step(F2, b, c, d, a, in[0] + 0xe9b6c7aa, 20);
-    thumb_md5_step(F2, a, b, c, d, in[5] + 0xd62f105d, 5);
-    thumb_md5_step(F2, d, a, b, c, in[10] + 0x02441453, 9);
-    thumb_md5_step(F2, c, d, a, b, in[15] + 0xd8a1e681, 14);
-    thumb_md5_step(F2, b, c, d, a, in[4] + 0xe7d3fbc8, 20);
-    thumb_md5_step(F2, a, b, c, d, in[9] + 0x21e1cde6, 5);
-    thumb_md5_step(F2, d, a, b, c, in[14] + 0xc33707d6, 9);
-    thumb_md5_step(F2, c, d, a, b, in[3] + 0xf4d50d87, 14);
-    thumb_md5_step(F2, b, c, d, a, in[8] + 0x455a14ed, 20);
-    thumb_md5_step(F2, a, b, c, d, in[13] + 0xa9e3e905, 5);
-    thumb_md5_step(F2, d, a, b, c, in[2] + 0xfcefa3f8, 9);
-    thumb_md5_step(F2, c, d, a, b, in[7] + 0x676f02d9, 14);
-    thumb_md5_step(F2, b, c, d, a, in[12] + 0x8d2a4c8a, 20);
-		
-    thumb_md5_step(F3, a, b, c, d, in[5] + 0xfffa3942, 4);
-    thumb_md5_step(F3, d, a, b, c, in[8] + 0x8771f681, 11);
-    thumb_md5_step(F3, c, d, a, b, in[11] + 0x6d9d6122, 16);
-    thumb_md5_step(F3, b, c, d, a, in[14] + 0xfde5380c, 23);
-    thumb_md5_step(F3, a, b, c, d, in[1] + 0xa4beea44, 4);
-    thumb_md5_step(F3, d, a, b, c, in[4] + 0x4bdecfa9, 11);
-    thumb_md5_step(F3, c, d, a, b, in[7] + 0xf6bb4b60, 16);
-    thumb_md5_step(F3, b, c, d, a, in[10] + 0xbebfbc70, 23);
-    thumb_md5_step(F3, a, b, c, d, in[13] + 0x289b7ec6, 4);
-    thumb_md5_step(F3, d, a, b, c, in[0] + 0xeaa127fa, 11);
-    thumb_md5_step(F3, c, d, a, b, in[3] + 0xd4ef3085, 16);
-    thumb_md5_step(F3, b, c, d, a, in[6] + 0x04881d05, 23);
-    thumb_md5_step(F3, a, b, c, d, in[9] + 0xd9d4d039, 4);
-    thumb_md5_step(F3, d, a, b, c, in[12] + 0xe6db99e5, 11);
-    thumb_md5_step(F3, c, d, a, b, in[15] + 0x1fa27cf8, 16);
-    thumb_md5_step(F3, b, c, d, a, in[2] + 0xc4ac5665, 23);
-		
-    thumb_md5_step(F4, a, b, c, d, in[0] + 0xf4292244, 6);
-    thumb_md5_step(F4, d, a, b, c, in[7] + 0x432aff97, 10);
-    thumb_md5_step(F4, c, d, a, b, in[14] + 0xab9423a7, 15);
-    thumb_md5_step(F4, b, c, d, a, in[5] + 0xfc93a039, 21);
-    thumb_md5_step(F4, a, b, c, d, in[12] + 0x655b59c3, 6);
-    thumb_md5_step(F4, d, a, b, c, in[3] + 0x8f0ccc92, 10);
-    thumb_md5_step(F4, c, d, a, b, in[10] + 0xffeff47d, 15);
-    thumb_md5_step(F4, b, c, d, a, in[1] + 0x85845dd1, 21);
-    thumb_md5_step(F4, a, b, c, d, in[8] + 0x6fa87e4f, 6);
-    thumb_md5_step(F4, d, a, b, c, in[15] + 0xfe2ce6e0, 10);
-    thumb_md5_step(F4, c, d, a, b, in[6] + 0xa3014314, 15);
-    thumb_md5_step(F4, b, c, d, a, in[13] + 0x4e0811a1, 21);
-    thumb_md5_step(F4, a, b, c, d, in[4] + 0xf7537e82, 6);
-    thumb_md5_step(F4, d, a, b, c, in[11] + 0xbd3af235, 10);
-    thumb_md5_step(F4, c, d, a, b, in[2] + 0x2ad7d2bb, 15);
-    thumb_md5_step(F4, b, c, d, a, in[9] + 0xeb86d391, 21);
-
-    buf[0] += a;
-    buf[1] += b;
-    buf[2] += c;
-    buf[3] += d;
-}
